@@ -9,7 +9,7 @@ import adafruit_pioasm
 
 squarewave = """
 .program squarewave
-    set pins 1 [1]  ; Drive pin high and then delay for one cycle
+    set pins 1      ; Drive pin high and then delay for one cycle
     set pins 0      ; Drive pin low
 """
 
@@ -17,9 +17,8 @@ assembled = adafruit_pioasm.assemble(squarewave)
 
 sm = rp2pio.StateMachine(
     assembled,
-    frequency=80,
-    init=adafruit_pioasm.assemble("set pindirs 1"),
-    first_set_pin=board.LED,
+    frequency=1000 * 2,
+    first_set_pin=board.D13,
 )
 print("real frequency", sm.frequency)
 
