@@ -18,7 +18,7 @@ def nice_opcode(o):
     return o[:3] + "_" + o[3:8] + "_" + o[8:]
 
 
-class TestNop(unittest.TestCase):
+class AssembleChecks(unittest.TestCase):
     def assertAssemblesTo(self, source, expected):
         actual = adafruit_pioasm.assemble(source)
         expected_bin = [nice_opcode(x) for x in expected]
@@ -35,6 +35,12 @@ class TestNop(unittest.TestCase):
         else:
             self.assertRaises(errtype, adafruit_pioasm.assemble, source)
 
+    def assertPioKwargs(self, source, **kw):
+        program = adafruit_pioasm.Program(source)
+        self.assertEqual(kw, program.pio_kwargs)
+
+
+class TestNop(AssembleChecks):
     def testNonsense(self):
         self.assertAssemblyFails("nope")
 
@@ -44,8 +50,9 @@ class TestNop(unittest.TestCase):
             "nop\nnop", [0b101_00000_010_00_010, 0b101_00000_010_00_010]
         )
         self.assertAssemblesTo("nop [1]", [0b101_00001_010_00_010])
+        self.assertAssemblesTo("nop [31]", [0b101_11111_010_00_010])
         self.assertAssemblesTo(".side_set 1\nnop side 1", [0b101_10000_010_00_010])
-        self.assertAssemblesTo(".side_set 1\nnop side 1 [1]", [0b101_10001_010_00_010])
+        self.assertAssemblesTo(".side_set 1\nnop side 1 [15]", [0b101_11111_010_00_010])
 
     def testSidesetOpt(self):
         self.assertAssemblesTo(".side_set 1 opt\nnop side 1", [0b101_11000_010_00_010])
@@ -54,11 +61,12 @@ class TestNop(unittest.TestCase):
             ".side_set 1 opt\nnop side 0 [1]", [0b101_10001_010_00_010]
         )
         self.assertAssemblesTo(".side_set 1 opt\nnop [1]", [0b101_00001_010_00_010])
-
-    def testMov(self):
-        # non happy path
-        self.assertAssemblyFails(
-            "mov x, blah", match="Invalid mov source 'blah'", errtype=ValueError
+        self.assertAssemblesTo(".side_set 1 opt\nnop [7]", [0b101_00111_010_00_010])
+        self.assertAssemblesTo(
+            ".side_set 1 opt\nnop side 1 [1]", [0b101_11001_010_00_010]
+        )
+        self.assertAssemblesTo(
+            ".side_set 1 opt\nnop side 0 [7]", [0b101_10111_010_00_010]
         )
 
     def testSet(self):
@@ -94,3 +102,60 @@ class TestNop(unittest.TestCase):
         self.assertAssemblesTo("wait 0 irq 0 rel", [0b001_00000_0_10_10000])
         self.assertAssemblesTo("wait 1 irq 0", [0b001_00000_1_10_00000])
         self.assertAssemblesTo("wait 0 irq 1 rel", [0b001_00000_0_10_10001])
+
+    def testLimits(self):
+        self.assertAssemblyFails(".side_set 1\nnop side 2")
+        self.assertAssemblyFails(".side_set 1\nnop side 2 [1]")
+        self.assertAssemblyFails("nop [32]")
+        self.assertAssemblyFails(".side_set 1\nnop side 0 [16]")
+        self.assertAssemblyFails(".side_set 1 opt\nnop side 0 [8]")
+
+    def testCls(self):
+        self.assertPioKwargs("", sideset_enable=False)
+        self.assertPioKwargs(".side_set 1", sideset_pin_count=1, sideset_enable=False)
+        self.assertPioKwargs(
+            ".side_set 3 opt", sideset_pin_count=3, sideset_enable=True
+        )
+
+
+class TestMov(AssembleChecks):
+    def testMovNonHappy(self):
+        # non happy path
+        self.assertAssemblyFails(
+            "mov x, blah", match="Invalid mov source 'blah'", errtype=ValueError
+        )
+
+    def testMovInvert(self):
+        # test moving and inverting
+        self.assertAssemblesTo("mov x, ~ x", [0b101_00000_001_01_001])
+        self.assertAssemblesTo("mov x, ~x", [0b101_00000_001_01_001])
+        self.assertAssemblesTo("mov x, !x", [0b101_00000_001_01_001])
+
+    def testMovReverse(self):
+        # test moving and reversing bits
+        self.assertAssemblesTo("mov x, :: x", [0b101_00000_001_10_001])
+        self.assertAssemblesTo("mov x, ::x", [0b101_00000_001_10_001])
+
+
+class TestWrap(AssembleChecks):
+    def testWrap(self):
+        self.assertAssemblyFails(".wrap")
+        self.assertPioKwargs(
+            "nop\n.wrap_target\nnop\nnop\n.wrap",
+            sideset_enable=False,
+            wrap=2,
+            wrap_target=1,
+        )
+
+
+class TestRadix(AssembleChecks):
+    def testOctal(self):
+        self.assertAssemblesTo(".side_set 0o1\nset x, 0o11", [0b111_00000_001_01001])
+
+    def testBinary(self):
+        self.assertAssemblesTo(
+            ".side_set 0b101\nnop side 0b10001", [0b101_10001_010_00_010]
+        )
+
+    def testHex(self):
+        self.assertAssemblesTo(".side_set 0x0\nnop [0x10]", [0b101_10000_010_00_010])
